;; ##################################################################################
;; # ============================================================================== #
;; # MrEd Designer - files.ss                                                       #
;; # http://mreddesigner.lozi.org                                                   #
;; # Copyright (C) Lozi Jean-Pierre, 2004 - mailto:jean-pierre@lozi.org             #
;; # ============================================================================== #
;; #                                                                                #
;; # This program is free software; you can redistribute it and/or                  #
;; # modify it under the terms of the GNU General Public License                    #
;; # as published by the Free Software Foundation; either version 2                 #
;; # of the License, or (at your option) any later version.                         #
;; #                                                                                #
;; # This program is distributed in the hope that it will be useful,                #
;; # but WITHOUT ANY WARRANTY; without even the implied warranty of                 #
;; # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                  #
;; # GNU General Public License for more details.                                   #
;; #                                                                                #
;; # You should have received a copy of the GNU General Public License              #
;; # along with this program; if not, write to the Free Software                    #
;; # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.    #
;; #                                                                                #
;; ##################################################################################

;; Function called when the user chooses "File->New"
(define (files-new)
  (if (not no-change-since-last-save)
      (begin
        (if (eq? 'yes (message-box "Warning" "Do you want to save the current project before opening a new one?" #f '(yes-no caution)))
            (files-save))))
  (preview-update-preview project-data-global '((project (label "Untitled Project")
                                                         (path "")
                                                         (selected #f)
                                                         (collapsed #f))))
  (send hierarchy-widget update-in)
  (send properties-widget update-in)
  (send properties-widget empty)
  (set! no-change-since-last-save #t))

;; Function called when a new file is to be loaded  -> 2004-11-27 Nicolas Dufresne <n.dufresne@usherbrooke.ca>
(define (use-file file-name)
      (if (string? file-name)
              (if (file-exists? file-name)
                  (call-with-input-file file-name
                    (lambda (p-in)
                      (if (equal? (read p-in) '(*med-file*))
                          (begin
                            (let ((new-data (read p-in)))
                              (preview-update-preview project-data-global new-data)
                              ;; We have to update the label and the path...
                              (set! project-data-global `((,(caar project-data-global)
                                                            (label ,@(let-values (((base name must-be-dir?)(split-path file-name)))
                                                                       (regexp-match "[^.]+" name)))
                                                            (path ,file-name) 
                                                            ,@(remq (assq 'label (cdar project-data-global))
                                                                    (cdar project-data-global)))))
                              (send hierarchy-widget update-in)
                              (send properties-widget update-in)
                              (set! no-change-since-last-save #t)))
                          (message-box "Error" "The file format is not valid." #f '(ok stop)))))
                  (message-box "Error" "The file does not exist." #f '(ok stop)))))

;; Function called when the user chooses "File->Open"
(define (files-open)
  (let ((result (if (not no-change-since-last-save)
                    (message-box/custom "Warning" 
                                        "Do you want to save the current project before opening a new one?"
                                        "Yes" "No" "Cancel" #f '(default=1 number-order caution) 3)
                    2)))
    (if (= 1 result) (files-save))
    (if (not (= 3 result))
        (let ((result (get-file "Open..." #f #f #f "*.med" '() '(("MrEd Designer Project (.med)" "*.med")))))
          (if result
              (use-file result))))))
  
;; Function called when the user chooses "File->Save As..."
(define (files-save-as)
  ;; Removes unnecessary tags from project-data-global
  (define (parse-line L)
    (cond ((null? L) '())
          ((symbol? (car L)) (cons (car L) (parse-line (cdr L))))
          ((eq? (caar L) 'pointer) (parse-line (cdr L)))
          ((eq? (caar L) 'path) (cons (list 'path "") (parse-line (cdr L))))
          (else (cons (car L) (parse-line (cdr L))))))
  ;; Function body
  (let ((result (put-file "Save as..." #f #f #f "*.med" '() '(("MrEd Designer Project (.med)" "*.med")))))
    (if result
        (begin
          (call-with-output-file result 
            (lambda (p-out)
              (fprintf p-out "~s\n" '(*med-file*))
              (pretty-print (apply-function-on-data-list project-data-global parse-line) p-out))
            'replace)
          ;; Then we update the no-change-since-last-save variable...
          (set! no-change-since-last-save #t)
          ;; We update the project label...
          (let-values (((base name must-be-dir?)(split-path result)))
            (project-data-set-label (car (regexp-match "[^.]+" name))))
          ;; We update the project path...
          (project-data-set-path result)))
    result))

;; Function called when the user chooses "File->Save"
(define (files-save)
  ;; Removes unnecessary tags from project-data-global
  (define (parse-line L)
    (cond ((null? L) '())
          ((symbol? (car L)) (cons (car L) (parse-line (cdr L))))
          ((eq? (caar L) 'pointer) (parse-line (cdr L)))
          ((eq? (caar L) 'path) (cons (list 'path "") (parse-line (cdr L))))
          (else (cons (car L) (parse-line (cdr L))))))
  ;; Function body
  (let ((path (project-data-get-path)))
    (if (not (string=? path ""))
        (if (file-exists? path)
            (begin
              (call-with-output-file path
                (lambda (p-out)
                  (fprintf p-out "~s\n" '(*med-file*))
                  (pretty-print (apply-function-on-data-list project-data-global parse-line) p-out))
                'replace)
              ;; Then we update the no-change-since-last-save variable...
              (set! no-change-since-last-save #t))
            (begin
              (message-box "Error" "Unable to save the project because the file does not exist anymore. Please check it has not been either deleted, moved or renamed." #f '(ok stop))
              (files-save-as)))
        (files-save-as))))

;; Function called when the user chooses "File->Quit"
(define (files-quit cancel)
    (let ((result (if (not no-change-since-last-save)
                      (if cancel
                          (message-box/custom "Warning" 
                                              "Would you like to save changes before quitting?"
                                              "Yes" "No" "Cancel" #f '(default=1 number-order caution) 3)
                          (message-box/custom "Warning" 
                                              "Would you like to save changes before quitting?"
                                              "Yes" "No" #f #f '(default=1 number-order caution) 3))
                    2)))
      (cond ((= 1 result) (if cancel
                              (if (files-save) (exit))
                              (begin (files-save) (exit))))
            ((= 2 result) (exit)))))
