;; ##################################################################################
;; # ============================================================================== #
;; # MrEd Designer - hierarchy-frame.ss                                             #
;; # http://mreddesigner.lozi.org                                                   #
;; # Copyright (C) Lozi Jean-Pierre, 2004 - mailto:jean-pierre@lozi.org             #
;; # ============================================================================== #
;; #                                                                                #
;; # This program is free software; you can redistribute it and/or                  #
;; # modify it under the terms of the GNU General Public License                    #
;; # as published by the Free Software Foundation; either version 2                 #
;; # of the License, or (at your option) any later version.                         #
;; #                                                                                #
;; # This program is distributed in the hope that it will be useful,                #
;; # but WITHOUT ANY WARRANTY; without even the implied warranty of                 #
;; # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the                  #
;; # GNU General Public License for more details.                                   #
;; #                                                                                #
;; # You should have received a copy of the GNU General Public License              #
;; # along with this program; if not, write to the Free Software                    #
;; # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.    #
;; #                                                                                #
;; ##################################################################################

;; We need to derive the frame class to override the on-close method.
(define hierarchy-frame%
  (class frame%
    ;; We override on-close...
    (override on-close)
    ;; When on-close is called, we exit the project.
    (define (on-close) (send properties-frame-windows-hierarchy-frame-menu-item check #f))
    (super-instantiate ())))

;; We instantiate the frame...
(define hierarchy-frame (new hierarchy-frame%
                             (label "Hierarchy")
                             (parent toolbox-frame)
                             (x 5)
                             (y 245)
                             (width (send toolbox-frame get-width))))

;; ...the main horizontal pane...
(define hierarchy-horizontal-panel (new horizontal-panel%
                                        (border 0)
                                        (parent hierarchy-frame)))

;; ...the buttons' panel...
(define buttons-panel (new vertical-panel%
                           (border 1)
                           (alignment '(center top))
                           (stretchable-width #f)
                           (parent hierarchy-horizontal-panel)))

;; ...the up button...
(define hierarchy-up-button (new button% 
                                 (label (make-object bitmap% (build-path "images" "hierarchy-up.png") 'png (make-object color% 255 255 255)))
                                 (callback (lambda (button control-event)
                                             (send hierarchy-widget swap-up-selected)
                                             (send hierarchy-widget update-buttons)))
                                 (enabled #f)
                                 (parent buttons-panel)))

;; ...the down button...
(define hierarchy-down-button (new button% 
                                     (label (make-object bitmap% (build-path "images" "hierarchy-down.png") 'png (make-object color% 255 255 255)))
                                     (callback (lambda (button control-event)
                                                 (send hierarchy-widget swap-down-selected)
                                                 (send hierarchy-widget update-buttons)))
                                     (enabled #f)
                                     (parent buttons-panel)))

;; ...the delete button...
(define hierarchy-delete-button (new button% 
                                     (label (make-object bitmap% (build-path "images" "hierarchy-delete.png") 'png (make-object color% 255 255 255)))
                                     (callback (lambda (button control-event)
                                                 (send hierarchy-widget delete-selected)
                                                 (send hierarchy-widget update-buttons)))
                                     (enabled #f)
                                     (parent buttons-panel)))
  
;; ...the hide button...
(define hierarchy-hide-button (new button% 
                                     (label (make-object bitmap% (build-path "images" "hierarchy-hide.png") 'png (make-object color% 255 255 255)))
                                     (callback (lambda (button control-event)
                                                 (let ((selected-element (send hierarchy-widget get-selected-element)))
                                                   (if selected-element
                                                       (begin
                                                         (send (cadr (assq 'pointer (cdr selected-element))) show #f)
                                                         (send hierarchy-widget update-buttons))))))
                                     (enabled #f)
                                     (parent buttons-panel)))

;; ...the show button...
(define hierarchy-show-button (new button% 
                                     (label (make-object bitmap% (build-path "images" "hierarchy-show.png") 'png (make-object color% 255 255 255)))
                                     (callback (lambda (button control-event)
                                               (let ((selected-element (send hierarchy-widget get-selected-element)))
                                                 (if selected-element
                                                     (begin
                                                       (send (cadr (assq 'pointer (cdr selected-element))) show #t)
                                                       (send hierarchy-widget update-buttons))))))
                                     (enabled #f)
                                     (parent buttons-panel)))

;; ...and the widget itself.
(define hierarchy-widget (new hierarchy-widget% (parent hierarchy-horizontal-panel)))
  
;; We want to be sure that the frame's virtual size is valid before calling update-in...
(send hierarchy-frame reflow-container)

;; We update the widget data... 
(send hierarchy-widget update-in)

;; We show the hierarchy frame...
(send hierarchy-frame show #t)

;; And then we send a show event to the toolbox frame so that it is focused on startup...
(send toolbox-frame show #t)
